import 'dart:io';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:ssl_pinning_example/network/ssl_pinning_manager.dart';
import 'package:ssl_pinning_example/network/url_constants.dart';

import 'mi_loader.dart';
import 'network/base_client.dart';
import 'network/fetch_exception.dart';

GlobalKey<NavigatorState> navigatorKey = GlobalKey<NavigatorState>();

void main() async {
  WidgetsFlutterBinding.ensureInitialized();
  await SSLPinningManger().initialize();
  runApp(const MyApp());
}

class MyApp extends StatelessWidget {
  const MyApp({super.key});

  // This widget is the root of your application.
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Flutter Demo',
      navigatorKey: navigatorKey,
      theme: ThemeData(
        colorScheme: ColorScheme.fromSeed(seedColor: Colors.deepPurple),
        useMaterial3: true,
      ),
      home: HomeScreen(
        controller: Get.put(HomeController()),
      ),
    );
  }
}

class HomeScreen extends StatelessWidget {
  const HomeScreen({super.key, required this.controller});

  final HomeController controller;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: AppBar(
          title: const Text("SSL Pinning"),
        ),
        body: Obx(
          () => MILoader(
            isCallInProgress: controller.isLoading.value,
            child: Center(
                child: ElevatedButton(
              onPressed: () => {controller.callAPI()},
              child: const Text('Call API'),
            )),
          ),
        ));
  }
}

class HomeController extends GetxController {
  var isLoading = false.obs;

  @override
  void onInit() {
    super.onInit();
  }

  callAPI() async {
    isLoading.value = true;
    await BaseClient.instance
        .get(UrlConstants.getCountries)
        .then((value) async {
      debugPrint(value);
      isLoading.value = false;
      _showSnackBar("API Success");
    }).onError((error, stackTrace) {
      isLoading.value = false;
      debugPrint(FetchException(error).fetchErrorModel().message ?? "");
      _showSnackBar(FetchException(error).fetchErrorModel().message ??
          "Something went wrong, please try again later");
    });
  }

  _showSnackBar(String message) {
    var snackBar = SnackBar(content: Text(message));
    ScaffoldMessenger.of(navigatorKey.currentContext!).showSnackBar(snackBar);
  }
}
