import 'package:http/http.dart' as http;
import 'package:ssl_pinning_example/network/error_model.dart';
import 'url_constants.dart';
import 'package:flutter/material.dart';
import 'dart:convert';
import 'app_exception.dart';

class BaseClient {
  static const int timeOutDuration = 60;
  final Map<String, String> _header = {
    "Content-Type": "application/json; charset=utf-8",
    "Accept-Language": "en-US",
  };

  BaseClient._internal();

  static final BaseClient _instance = BaseClient._internal();

  static BaseClient get instance => _instance;

  late final http.Client client;

  factory BaseClient(http.Client client) {
    _instance.client = client;
    return _instance;
  }

  //GET
  Future<dynamic> get(String api) async {
    var uri = Uri.parse(UrlConstants.baseUrl + api);

    debugPrint(uri.toString());
    debugPrint(_header.toString());
    try {
      var response = await client
          .get(uri, headers: _header)
          .timeout(const Duration(seconds: timeOutDuration));
      return _processResponse(response);
    } on GeneralApiResponseErrorException catch (error) {
      throw GeneralApiResponseErrorException(error.errorModel);
    } on Exception {
      throw GeneralApiResponseErrorException(getDefaultErrorResponse());
    }
  }

  dynamic _processResponse(http.Response response) {
    debugPrint("response code ${utf8.decode(response.bodyBytes)}");
    if (response.isOkResponse()) {
      debugPrint("API Success");
      return utf8.decode(response.bodyBytes);
    } else if (response.isUnauthorizedUser()) {
      debugPrint("StatusCode---${response.statusCode}");
      getErrorModel(response);
    } else {
      debugPrint("API Response is Not Ok");
      getErrorModel(response);
    }
  }

  getErrorModel(http.Response response) {
    var responseBytes = utf8.decode(response.bodyBytes);
    var errorModel = ErrorModel.fromJson(jsonDecode(responseBytes));
    debugPrint(errorModel.toJson().toString());
    if (errorModel.message != null) {
      throw GeneralApiResponseErrorException(errorModel.toRawJson().toString());
    } else {
      throw GeneralApiResponseErrorException(getDefaultErrorResponse());
    }
  }

  String getDefaultErrorResponse() {
    String defaultValue =
        (ErrorModel(message: "Something went wrong, please try again later."))
            .toRawJson();
    return defaultValue.toString();
  }
}

extension on http.Response {
  bool isOkResponse() {
    return statusCode >= 200 && statusCode <= 299;
  }

  bool isUnauthorizedUser() {
    return statusCode == 401;
  }
}
